'==================================================================='
'			 LEGOCNTL.BAS
'==================================================================='
'
' A program to control LEGO using the Parallel port
'
' WRITTEN BY : MIKE GENOVY
'
' DATE : JULY 28, 1998
'
' LAST MODIFIED: AUGUST 9, 1998
'                SEPTEMBER 2, 1998
' JANUARY 3, 2000 - updated code to display input bits in Manual Mode
'
'
'
'
'-------------------------------------------------------------------

CLS			'CLEAR SCREEN FIRST

'-------------------------------------------------------------------
' SET UP THE SCREEN COLORS
'-------------------------------------------------------------------

COLOR 7

'-------------------------------------------------------------------
' DISPLAY PROGRAM INFORMATION
'
'-------------------------------------------------------------------

GOSUB SIGNON

'=====================================================================
' FIRST DISPLAY THE FUNCTION KEY MENU AT THE BOTTOM OF THE SCREEN
'
' THE STANDARD DEFAULT KEY ASSIGNMENTS ARE USED. THESE ARE THE FUNCTION
' KEYS F1 TO F10 AT THE TOP OF THE KEYBOARD.
'=====================================================================
'
'DEFINE KEY LABELS FOR DISPLAY ON LINE 25
'
KEY 1, " RUN "          'KEY 1  - F1
KEY 2, " LIST "         'KEY 2  - F2
KEY 3, " PRGM "         'KEY 3  - F3
KEY 4, " TEST "         'KEY 4  - F4
KEY 5, " MAN "          'KEY 5  - F5
KEY 6, " XTRA "         'KEY 6  - F6
KEY 7, " XTRA "         'KEY 7  - F7
KEY 8, " XTRA "         'KEY 8  - F8
KEY 9, " HELP "        'KEY 9  - F9
KEY 10, " EXIT "        'KEY 10 - F10

KEY ON				'DISPLAY KEY LABELS ON LINE 25

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON

'----------------------------------------------------------------------
' NOW CHECK IF KEY IS PRESSED - IF SO GO TO SUBROUTINE
'----------------------------------------------------------------------

2  ON KEY(1) GOSUB RUNLEGO
   ON KEY(2) GOSUB LISTLEGO
   ON KEY(3) GOSUB PROGRAMLEGO
   ON KEY(4) GOSUB TESTPORT
   ON KEY(5) GOSUB MANUALOUTPUT
   ON KEY(6) GOSUB SPAREKEY
   ON KEY(7) GOSUB SPAREKEY
   ON KEY(8) GOSUB SPAREKEY
   ON KEY(9) GOSUB HELP
   ON KEY(10) GOSUB QUIT

   GOTO 2

END

'STOP

'======================================================================
'			 SIGNON SUBROUTINE
'======================================================================

SIGNON:

CLS

LOCATE 2,20
PRINT "+============================================+"
'LOCATE 2,20
'PRINT "|                                            |"
LOCATE 3,20
PRINT "|      LEGO CONTROL PROGRAM (LEGOCNTL.EXE)   |"
'LOCATE 4,20
'PRINT "|      ...................................   |"
'LOCATE 5,20
'PRINT "|                                            |"
LOCATE 4,20
PRINT "|               Version 1.00                 |"
LOCATE 5,20
PRINT "|         By: M.Genovy & J.Genovy            |"
LOCATE 6,20
PRINT "|              January 3, 2000               |"
LOCATE 7,20
PRINT "|                                            |"
LOCATE 8,20
PRINT "+============================================+"

LOCATE 10,1
PRINT " FUNCTION KEY             DESCRIPTION "
PRINT " --------------------------------------------------------"
PRINT
PRINT " F1  = RUN   -  Run a saved LEGO program "
PRINT " F2  = LIST  -  Display a LEGO program on screen "
PRINT " F3  = PRGM  -  Enter and save a LEGO program "
PRINT " F4  = TEST  -  Test the Parallel Port "
PRINT " F5  = MAN   -  Output LEGO commands manually "
PRINT " F6  = XTRA  -  Spare Function key "
PRINT " F7  = XTRA  -  Spare Function key "
PRINT " F8  = XTRA  -  Spare Function key "
PRINT " F9  = HELP  -  Display detailed instructions on program "
PRINT " F10 = EXIT  -  Quit this program "

RETURN

'======================================================================
'		***** RUNLEGO SUBROUTINE *****
'======================================================================

RUNLEGO:
	PRINT "RUNLEGO SUBROUTINE"

'---------------------------------------------------------------------
' TURN KEY CHECKING OFF
'---------------------------------------------------------------------

KEY OFF

KEY (1) OFF			'TURN OFF KEY CHECKING
KEY (2) OFF
KEY (3) OFF
KEY (4) OFF
KEY (5) OFF
KEY (6) OFF
KEY (7) OFF
KEY (8) OFF
KEY (9) OFF
KEY (10) OFF

'----------------------------------------------------------------------
' FIRST DISPLAY THE DIRECTORY OF SAVED LEGO PROGRAMS
' PROGRAMS MUST BE SAVED IN A FILE WITH A .LEG EXTENSION
'----------------------------------------------------------------------

CLS
FILES "*.LEG"

'----------------------------------------------------------------------
' GET NAME OF SAVED LEGO PROGRAM TO RUN
'----------------------------------------------------------------------

GETFILENAME:

INPUT "ENTER LEGO PROGRAM FILE TO RUN ( ENTER FILE EXTENSION ALSO ! ) ",FILENAME$

IF FILENAME$ = "" GOTO GETFILENAME

INPUT "DO YOU WANT TO SINGLE STEP THE PROGRAM ? (Y/N) ",SINGLESTEP$

'----------------------------------------------------------------------
' OPEN THE SAVED LEGO PROGRAM FILE FOR INPUT
'----------------------------------------------------------------------

OPEN FILENAME$ FOR INPUT AS #1

'----------------------------------------------------------------------
' READ A LINE FROM THE FILE AND PROCESS THE COMMAND
' CONTINUE UNTIL THE END OF FILE IS ENCOUNTERED
'----------------------------------------------------------------------

LINENUMBER% = 0				'RESET LINENUMBER COUNTER
SKIP% = 0				'RESET SKIP LINE COUNTER

WHILE NOT EOF(1)

'----------------------------------------------------------------------
' CHECK THE SKIP COUNTER TO SEE IF WE NEED TO SKIP ANY LINES IN FILE
'----------------------------------------------------------------------

SKIP:

  IF SKIP% = 0 GOTO NOSKIP		'CHECK SKIP LINE COUNTER

     LINE INPUT #1,TEMPLINE$		'READ IN A LINE
     SKIP% = SKIP% - 1			'DECREMENT SKIP LINE COUNTER
     GOTO SKIP                          'GO BACK AND SEE IF MORE LINES TO SKIP

'----------------------------------------------------------------------
' CHECK THE SINGLE STEP FLAG TO SEE IF WE MUST SINGLE STEP
'----------------------------------------------------------------------

NOSKIP:

  IF SINGLESTEP$ = "N" OR SINGLESTEP$ = "n" GOTO READLINE
  INPUT " PRESS ANY KEY TO EXECUTE NEXT LINE IN PROGRAM FILE ",A$

READLINE:

  LINE INPUT #1,TEMPLINE$		'READ IN A LINE

  PRINT TEMPLINE$                       'PRINT TO SCREEN FOR TESTING
  LINENUMBER% = LINENUMBER% + 1

'----------------------------------------------------------------------
' DETERMINE WHAT COMMAND IS TO BE EXECUTED BY SEARCHING STRING FOR KEY
' WORDS. THE KEY WORDS ARE LISTED BELOW:
'----------------------------------------------------------------------
'
' ON #            - ON COMMAND
' OFF #           - OFF COMMAND
' WAIT #          - WAIT FOR # SECONDS
' IF SW # ON *    - IF SWITCH INPUT # ON THEN TURN ON OUTPUT *
' IF SW # OFF *   - IF SWITCH INPUT # ON THEN TURN OFF OUTPUT *
' IF SW # SKIP X  - IF SWITCH INPUT # ON THEN SKIP X LINES
' SKIP X          - SKIP X LINES
' END             - THE END OF THE LEGO PROGRAM FILE
'
'----------------------------------------------------------------------
'
'SEE IF THIS IS A COMMENT LINE - IF SO SKIP IT
'
IF INSTR(TEMPLINE$,"'") THEN GOTO NEXTLINE

IF INSTR(TEMPLINE$,"ON") THEN GOTO ONCMD
IF INSTR(TEMPLINE$,"OFF") THEN GOTO OFFCMD
IF INSTR(TEMPLINE$,"WAIT") THEN GOTO WAITCMD
IF INSTR(TEMPLINE$,"IF SW") THEN GOTO IFCMD
IF INSTR(TEMPLINE$,"SKIP") THEN GOTO SKIPCMD
IF INSTR(TEMPLINE$,"END") THEN GOTO ENDCMD

'----------------------------------------------------------------------
' VALID COMMAND NOT FOUND - OUTPUT ERROR MESSAGE
'----------------------------------------------------------------------

BEEP 2					'GET THE USERS ATTENTION FIRST

PRINT " INVALID COMMAND FOUND ON LINE NUMBER " LINENUMBER%
DELAY 5

GOTO NEXTLINE				'CONTINUE WITH NEXT LINE

'======================================================================
'		***** ON COMMAND FOUND *****	
'======================================================================

ONCMD:

A$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"ON")+3),1)

PRINT "ON COMMAND FOUND FOR BIT "A$	'FOR TESTING ONLY

'DELAY 5

'--------------------------------------------------------------------'
' TURN ON THE PORT LINE                                              '
' OR the new output with previous output value and also update the   '
' S$ string used to display the Port Line status on the screen       '
'--------------------------------------------------------------------'

   IF A$="1" THEN		'OR outputs to create new value
   OUT &H378,(1 OR C%)		'Output to Port
   C%=C% OR 1			'Update the previous value
   MID$(S$,10)="1"              'Update the S$ display string
ELSEIF A$="2" THEN              'Port Line 2
   OUT &H378,(2 OR C%)          'Output to Port
   C%=C% OR 2                   'Update the previous value
   MID$(S$,16)="1"              'Update the S$ display string
ELSEIF A$="3" THEN              'Port Line 3
   OUT &H378,(4 OR C%)
   C%=C% OR 4
   MID$(S$,22)="1"
ELSEIF A$="4" THEN              'Port Line 4
   OUT &H378,(8 OR C%)
   C%=C% OR 8
   MID$(S$,28)="1"
ELSEIF A$="5" THEN		'Port Line 5	
   OUT &H378,(16 OR C%)
   C%=C% OR 16
   MID$(S$,34)="1"
ELSEIF A$="6" THEN              'Port Line 6
   OUT &H378,(32 OR C%)
   C%=C% OR 32
   MID$(S$,40)="1"
ELSEIF A$="7" THEN              'Port Line 7
   OUT &H378,(64 OR C%)
   C%=C% OR 64
   MID$(S$,46)="1"
ELSEIF A$="8" THEN              'Port Line 8
   OUT &H378,(128 OR C%)
   C%=C% OR 128
   MID$(S$,52)="1"
END IF

GOTO NEXTLINE

'======================================================================
'		***** OFF COMMAND FOUND *****
'======================================================================

OFFCMD:

A$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"OFF")+4),1)

PRINT "OFF COMMAND FOUND FOR BIT "A$	'FOR TESTING ONLY

'DELAY 5

'--------------------------------------------------------------------'
' TURN OFF THE PORT LINE                                             '
' AND the new output with previous output value and also update the  '
' S$ string used to display the Port Line status on the screen       '
'--------------------------------------------------------------------'

   IF A$="1" THEN               'AND outputs to create new value
   OUT &H378,(254 AND C%)       'Output to Port
   C%=C% AND 254                'Update the previous value
   MID$(S$,10)="0"              'Update the S$ display string
ELSEIF A$="2" THEN              'Port Line 2
   OUT &H378,(253 AND C%)
   C%=C% AND 253
   MID$(S$,16)="0"
ELSEIF A$="3" THEN              'Port Line 3
   OUT &H378,(251 AND C%)
   C%=C% AND 251
   MID$(S$,22)="0"
ELSEIF A$="4" THEN              'Port Line 4
   OUT &H378,(247 AND C%)
   C%=C% AND 247
   MID$(S$,28)="0"
ELSEIF A$="5" THEN              'Port Line 5
   OUT &H378,(239 AND C%)
   C%=C% AND 239
   MID$(S$,34)="0"
ELSEIF A$="6" THEN              'Port Line 6
   OUT &H378,(223 AND C%)
   C%=C% AND 223
   MID$(S$,40)="0"
ELSEIF A$="7" THEN              'Port Line 7
   OUT &H378,(191 AND C%)
   C%=C% AND 191
   MID$(S$,46)="0"
ELSEIF A$="8" THEN              'Port Line 8
   OUT &H378,(127 AND C%)
   C%=C% AND 127
   MID$(S$,52)="0"
END IF

GOTO NEXTLINE

'======================================================================
'		*****  WAIT COMMAND FOUND *****
'======================================================================

WAITCMD:

PRINT "WAIT COMMAND FOUND "		'FOR TESTING ONLY

'----------------------------------------------------------------------
'GET 4 CHARACTER DELAY VALUE...SO YOU CAN HAVE DELAY FROM XX.XX TO XXXX
'----------------------------------------------------------------------

A$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"WAIT")+5),4)

DELAY VAL(A$)				'DELAY THE DESIRED AMOUNT

GOTO NEXTLINE

'======================================================================
'		***** IF COMMAND FOUND ****
'======================================================================
'
' THE IF COMMAND CAN HAVE THE FOLLOWING FORMATS:
' ----------------------------------------------
' IF SW # ON *    - IF SWITCH INPUT # ON THEN TURN ON OUTPUT *
' IF SW # OFF *   - IF SWITCH INPUT # ON THEN TURN OFF OUTPUT *
' IF SW # SKIP X  - IF SWITCH INPUT # ON THEN SKIP X LINES
'

IFCMD:

PRINT "IF COMMAND FOUND "		'FOR TESTING ONLY

INBIT$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"SW")+3),1)

IF INSTR(TEMPLINE$,"ON") GOTO ONIF

IF INSTR(TEMPLINE$,"OFF") GOTO OFFIF

IF INSTR(TEMPLINE$,"SKIP") GOTO SKIPIF

'----------------------------------------------------------------------
' VALID IF COMMAND NOT FOUND - OUTPUT ERROR MESSAGE
'----------------------------------------------------------------------

BEEP 2					'GET THE USERS ATTENTION FIRST

PRINT " INVALID COMMAND FOUND ON LINE NUMBER " LINENUMBER%
DELAY 5

GOTO NEXTLINE				'CONTINUE WITH NEXT LINE

'----------------------------------------------------------------------
' SKIP IF COMMAND FOUND
'----------------------------------------------------------------------

SKIPIF:

SKIP$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"SKIP")+5),1)

SKIP% = VAL(SKIP$)			'CONVERT TO INTEGER VALUE

GOTO NEXTLINE				'CONTINUE WITH NEXT LINE

'----------------------------------------------------------------------
' OFF IF COMMAND FOUND
'----------------------------------------------------------------------

OFFIF:

OUTBIT$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"OFF")+4),1)

'----------------------------------------------------------------------
' TEST TO SEE IF INBIT$ IS OFF - IF SO TURN ON OUTBIT$
'----------------------------------------------------------------------




GOTO NEXTLINE				'CONTINUE WITH NEXT LINE

'----------------------------------------------------------------------
' ON IF COMMAND FOUND
'----------------------------------------------------------------------

ONIF:

OUTBIT$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"ON")+3),1)

'----------------------------------------------------------------------
' TEST TO SEE IF INBIT$ IS ON - IF SO TURN ON OUTBIT$
'----------------------------------------------------------------------



GOTO NEXTLINE				'CONTINUE WITH NEXT LINE

'======================================================================
'		***** SKIP COMMAND FOUND *****
'======================================================================

SKIPCMD:

PRINT "SKIP COMMAND FOUND "		'FOR TESTING ONLY

SKIP$ = MID$(TEMPLINE$,(INSTR(TEMPLINE$,"SKIP")+5),1)

SKIP% = VAL(SKIP$)			'CONVERT TO INTEGER VALUE

GOTO NEXTLINE

'======================================================================
'		***** END COMMAND FOUND *****
'======================================================================

ENDCMD:

PRINT "END COMMAND FOUND "		'FOR TESTING ONLY

GOTO EXITRUN				'EXIT THIS ROUTINE



NEXTLINE:


WEND					'END OF FILE READ LOOP

DELAY 5

EXITRUN:

CLOSE 1					'CLOSE THE INPUT FILE

'---------------------------------------------------------------------
' TURN KEY CHECKING BACK ON
'---------------------------------------------------------------------

CLS

GOSUB SIGNON			'DISPLAY THE SIGNON SCREEN AGAIN

KEY ON

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON

RETURN

'======================================================================
'			LISTLEGO SUBROUTINE
'======================================================================

LISTLEGO:
	PRINT "LISTLEGO SUBROUTINE"

'---------------------------------------------------------------------
' TURN KEY CHECKING OFF
'---------------------------------------------------------------------

KEY OFF

KEY (1) OFF			'TURN OFF KEY CHECKING
KEY (2) OFF
KEY (3) OFF
KEY (4) OFF
KEY (5) OFF
KEY (6) OFF
KEY (7) OFF
KEY (8) OFF
KEY (9) OFF
KEY (10) OFF

'----------------------------------------------------------------------
' FIRST DISPLAY THE DIRECTORY OF SAVED LEGO PROGRAMS
'----------------------------------------------------------------------

CLS
FILES "*.LEG"

'----------------------------------------------------------------------
' GET NAME OF SAVED LEGO PROGRAM TO RUN
'----------------------------------------------------------------------

INPUT "ENTER THE LEGO PROGRAM FILE TO LIST (.LEG EXTENSION ) ",FILENAME$

'----------------------------------------------------------------------
' OPEN THE SAVED LEGO PROGRAM FILE FOR INPUT
'----------------------------------------------------------------------

OPEN FILENAME$ FOR INPUT AS #1

'----------------------------------------------------------------------
' READ A LINE FROM THE FILE AND DISPLAY TO SCREEN
' CONTINUE UNTIL THE END OF FILE IS ENCOUNTERED
'----------------------------------------------------------------------

LINENUMBER% = 0				'RESET LINENUMBER COUNTER

WHILE NOT EOF(1)

  LINE INPUT #1,TEMPLINE$		'READ IN A LINE

  PRINT TEMPLINE$                       'PRINT TO SCREEN FOR TESTING
  LINENUMBER% = LINENUMBER% + 1

'----------------------------------------------------------------------
' SEE IF SCREEN IS FULL
'----------------------------------------------------------------------

  IF LINENUMBER% < 23 GOTO NEXTLISTLINE
    INPUT "PRESS ANY KEY TO DISPLAY THE NEXT SCREEN",A$
    LINENUMBER% = 0

NEXTLISTLINE:

WEND					'END OF FILE READ LOOP

DELAY 5

EXITLIST:

CLOSE 1					'CLOSE THE INPUT FILE

'---------------------------------------------------------------------
' TURN KEY CHECKING BACK ON
'---------------------------------------------------------------------

CLS

GOSUB SIGNON			'DISPLAY THE SIGNON SCREEN AGAIN

KEY ON

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON


RETURN

'======================================================================
'			PROGRAMLEGO SUBROUTINE
'======================================================================

PROGRAMLEGO:
	PRINT "PROGRAMLEGO SUBROUTINE"

'---------------------------------------------------------------------
' TURN KEY CHECKING OFF
'---------------------------------------------------------------------

KEY OFF

KEY (1) OFF			'TURN OFF KEY CHECKING
KEY (2) OFF
KEY (3) OFF
KEY (4) OFF
KEY (5) OFF
KEY (6) OFF
KEY (7) OFF
KEY (8) OFF
KEY (9) OFF
KEY (10) OFF

'----------------------------------------------------------------------
' FIRST DISPLAY THE DIRECTORY OF SAVED LEGO PROGRAMS
'----------------------------------------------------------------------

CLS
FILES "*.LEG"

'----------------------------------------------------------------------
' GET NAME OF SAVED LEGO PROGRAM TO RUN
'----------------------------------------------------------------------

INPUT "ENTER THE LEGO PROGRAM FILE TO EDIT (.LEG EXTENSION ) ",FILENAME$

'---------------------------------------------------------------------
' SHELL OUT TO TED EDITOR
'---------------------------------------------------------------------

PARAM$ = "TED " + FILENAME$	'CREATE PARAMETER STRING
'PRINT PARAM$
'DELAY 5
'SHELL "TED FILENAME$"

SHELL PARAM$			'SHELL TO DOS AND PASS PARAMETER STRING

'---------------------------------------------------------------------
' TURN KEY CHECKING BACK ON
'---------------------------------------------------------------------

CLS

GOSUB SIGNON			'DISPLAY THE SIGNON SCREEN AGAIN

KEY ON

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON

RETURN

'======================================================================
'			TESTPORT SUBROUTINE
'======================================================================

TESTPORT:
	PRINT "TESTPORT SUBROUTINE"

'---------------------------------------------------------------------
' TURN KEY CHECKING OFF
'---------------------------------------------------------------------

KEY OFF

KEY (1) OFF			'TURN OFF KEY CHECKING
KEY (2) OFF
KEY (3) OFF
KEY (4) OFF
KEY (5) OFF
KEY (6) OFF
KEY (7) OFF
KEY (8) OFF
KEY (9) OFF
KEY (10) OFF

CLS

'--------------------------------------------------------------------'
' SET UP TEST COUNTER VALUE                                          '
'--------------------------------------------------------------------'

TESTCOUNT% = 0

'--------------------------------------------------------------------'
' GET THE DELAY VALUE FOR TEST                                       '
'--------------------------------------------------------------------'

GETDELAY:

  INPUT " ENTER THE DELAY BETWEEN COMMANDS ( 0.05 TO 5 SECONDS ) "A#
  IF A# < 0.05 GOTO GETDELAY
  IF A# > 5.00 GOTO GETDELAY

'--------------------------------------------------------------------'
' TURN OFF ALL THE PORT LINES                                        '
'--------------------------------------------------------------------'

WHILE NOT INSTAT

OUT &H378,0			'Turn off all Port Lines

C%=0				'Set up the Port Output Value

'--------------------------------------------------------------------'
' TURN ON THE PORT LINES ONE BIT AT A TIME                           '
' OR the new output with previous output value                       '
'--------------------------------------------------------------------'

   PRINT "-----------------------------------------------------------"
   PRINT " PRESS ANY KEY TO STOP TEST ROUTINE "
   PRINT "-----------------------------------------------------------"
   PRINT " Turning on Bit 1 "
   OUT &H378,(1 OR C%)		'Turn on Bit 1
   C%=C% OR 1			'Update the previous value
   DELAY A#			'Wait a bit
   PRINT " Turning on Bit 2 "
   OUT &H378,(2 OR C%)          'Turn on Bit 2
   C%=C% OR 2                   'Update the previous value
   DELAY A#
   PRINT " Turning on Bit 3 "
   OUT &H378,(4 OR C%)          'Turn on Bit 3
   C%=C% OR 4                   '
   DELAY A#
   PRINT " Turning on Bit 4 "
   OUT &H378,(8 OR C%)          'Turn on Bit 4
   C%=C% OR 8                   '
   DELAY A#
   PRINT " Turning on Bit 5 "
   OUT &H378,(16 OR C%)         'Turn on Bit 5
   C%=C% OR 16                  '
   DELAY A#
   PRINT " Turning on Bit 6 "
   OUT &H378,(32 OR C%)         'Turn on Bit 6
   C%=C% OR 32                  '
   DELAY A#
   PRINT " Turning on Bit 7 "
   OUT &H378,(64 OR C%)         'Turn on Bit 7
   C%=C% OR 64                  '
   DELAY A#
   PRINT " Turning on Bit 8 "
   OUT &H378,(128 OR C%)        'Turn on Bit 8
   C%=C% OR 128                 '

'--------------------------------------------------------------------'
' TURN OFF THE PORT LINES ONE BIT AT A TIME                          '
' AND the new output with previous output value                      '
'--------------------------------------------------------------------'
   PRINT "-----------------------------------------------------------"
   DELAY A#
   PRINT " Turning off Bit 1 "
   OUT &H378,(254 AND C%)       'Turn off Bit 1
   C%=C% AND 254                'Update the previous value
   DELAY A#			'Wait a bit
   PRINT " Turning off Bit 2 "
   OUT &H378,(253 AND C%)	'Turn off Bit 2
   C%=C% AND 253                '
   DELAY A#
   PRINT " Turning off Bit 3 "
   OUT &H378,(251 AND C%)       'Turn off Bit 3
   C%=C% AND 251                '
   DELAY A#
   PRINT " Turning off Bit 4 "
   OUT &H378,(247 AND C%)       'Turn off Bit 4
   C%=C% AND 247                '
   DELAY A#
   PRINT " Turning off Bit 5 "
   OUT &H378,(239 AND C%)       'Turn off Bit 5
   C%=C% AND 239                '
   DELAY A#
   PRINT " Turning off Bit 6 "
   OUT &H378,(223 AND C%)       'Turn off Bit 6
   C%=C% AND 223                '
   DELAY A#
   PRINT " Turning off Bit 7 "
   OUT &H378,(191 AND C%)        'Turn off Bit 7
   C%=C% AND 191                 '
   DELAY A#
   PRINT " Turning off Bit 8 "
   OUT &H378,(127 AND C%)       'Turn off Bit 8
   C%=C% AND 127                '

'---------------------------------------------------------------------
'INCREMENT TEST COUNT AND DISPLAY IT
'---------------------------------------------------------------------

   TESTCOUNT% = TESTCOUNT% + 1
   PRINT "=========================================="
   PRINT " TEST EXECUTED "TESTCOUNT%" TIMES. "
   PRINT "=========================================="

WEND

'---------------------------------------------------------------------
' TURN KEY CHECKING BACK ON
'---------------------------------------------------------------------

CLS

GOSUB SIGNON			'DISPLAY THE SIGNON SCREEN AGAIN

KEY ON				'DISPLAY KEY LABELS ON LINE 25

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON

RETURN

'======================================================================
'			MANUALOUTPUT SUBROUTINE
'======================================================================

MANUALOUTPUT:
	PRINT "MANUALOUTPUT SUBROUTINE"

'---------------------------------------------------------------------
' TURN KEY CHECKING OFF
'---------------------------------------------------------------------

KEY OFF

KEY (1) OFF			'TURN OFF KEY CHECKING
KEY (2) OFF
KEY (3) OFF
KEY (4) OFF
KEY (5) OFF
KEY (6) OFF
KEY (7) OFF
KEY (8) OFF
KEY (9) OFF
KEY (10) OFF
'
'OUTPUT STATUS STRING
'
S$="   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |"
S1$="--> Port Status"
'
'INPUT STATUS STRING
'
SI$="   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |"

'--------------------------------------------------------------------'
' TURN OFF ALL THE PORT LINES                                        '
'--------------------------------------------------------------------'

OUT &H378,0			'Turn off all Port Lines

C%=0				'Set up the Port Output Value

'----------------------------------------------------------------------
' READ THE INPUT PORT BASED ON THE TIMER AT 1 SECOND INTERVALS
'----------------------------------------------------------------------

ON TIMER (1) GOSUB READINPUTS

TIMER ON			'ENABLE THE TIMER

10 CLS		  		'Clear the screen and display the Title

PRINT"==============================================================================="
PRINT"                    PORT INPUT DATA LINE STATUS DISPLAY
PRINT"==============================================================================="
'
'Display the Port Input Data Line status on the screen. 0 = off 1 = on
'
PRINT
PRINT"   +-----+-----+-----+-----+-----+-----+-----+-----+"
PRINT"   |  1  |  2  |  3  |  4  |  5  |     |     |     | --> Port Input Data Lines"
PRINT"   +-----+-----+-----+-----+-----+-----+-----+-----+"
PRINT SI$,s1$
'PRINT"   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |"
PRINT"   +-----+-----+-----+-----+-----+-----+-----+-----+     0 = OFF 1 = ON"
'
'Display the Port Output Data Line status on the screen. 0 = off 1 = on
'
PRINT
PRINT
PRINT"==============================================================================="
PRINT"                    PORT OUTPUT DATA LINE STATUS DISPLAY
PRINT"==============================================================================="
PRINT
PRINT"   +-----+-----+-----+-----+-----+-----+-----+-----+"
PRINT"   |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  | --> Port Output Data Lines"
PRINT"   +-----+-----+-----+-----+-----+-----+-----+-----+"
PRINT S$,s1$
'PRINT"   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |"
PRINT"   +-----+-----+-----+-----+-----+-----+-----+-----+     0 = OFF 1 = ON"
PRINT
PRINT "Enter the Port Line to change (Q to quit,C to clear all lines) "

'----------------------------------------------------------------------------
'GET KEYBOARD INPUT WITHOUT WAITNG BY USING INKEY$ INSTEAD OF INPUT STATEMENT
'----------------------------------------------------------------------------

GETPORT:
  A$=INKEY$			'Get keyboard input without waiting
  LOCATE 22,63          	'Print cursor at end of prompt line
  PRINT "_"             	'
  IF A$="Q" GOTO 50     	'Check if the User wants to quit
  IF A$="q" GOTO 50
  IF A$="C" GOTO 40		'Check if user wants to Clear all lines		
  IF A$="c" GOTO 40		'Check if user wants to Clear all lines		

  IF A$="1" OR A$="2" OR A$="3" OR A$="4" GOTO PRINTBIT
  IF A$="5" OR A$="6" OR A$="7" OR A$="8" GOTO PRINTBIT

  GOTO GETPORT			'Input not correct so go back for more

PRINTBIT:

  LOCATE 22,64			'Position cursor
  PRINT A$    			'Print selected output bit
  LOCATE 23,1	          	'Print prompt line
  PRINT "Enter 1 to turn ON, 2 to turn OFF"

GETFUNCTION:
  B$=INKEY$			'Get keyboard input without waiting
  LOCATE 23,43	        	'Print cursor at end of prompt line
  PRINT "_"			'
  IF VAL(A$) > 8 GOTO 10 	'Make sure Port Line in correct range
  IF VAL(A$) < 1 GOTO 10
  IF B$="1" GOTO 20             'Check for ON or OFF function
  IF B$="2" GOTO 30
  GOTO GETFUNCTION		'Input not correct so go back for more

'--------------------------------------------------------------------'
' TURN ON THE PORT LINE                                              '
' OR the new output with previous output value and also update the   '
' S$ string used to display the Port Line status on the screen       '
'--------------------------------------------------------------------'

20 IF A$="1" THEN		'OR outputs to create new value
   OUT &H378,(1 OR C%)		'Output to Port
   C%=C% OR 1			'Update the previous value
   MID$(S$,7)="1"              'Update the S$ display string
ELSEIF A$="2" THEN              'Port Line 2
   OUT &H378,(2 OR C%)          'Output to Port
   C%=C% OR 2                   'Update the previous value
   MID$(S$,13)="1"              'Update the S$ display string
ELSEIF A$="3" THEN              'Port Line 3
   OUT &H378,(4 OR C%)
   C%=C% OR 4
   MID$(S$,19)="1"
ELSEIF A$="4" THEN              'Port Line 4
   OUT &H378,(8 OR C%)
   C%=C% OR 8
   MID$(S$,25)="1"
ELSEIF A$="5" THEN		'Port Line 5	
   OUT &H378,(16 OR C%)
   C%=C% OR 16
   MID$(S$,31)="1"
ELSEIF A$="6" THEN              'Port Line 6
   OUT &H378,(32 OR C%)
   C%=C% OR 32
   MID$(S$,37)="1"
ELSEIF A$="7" THEN              'Port Line 7
   OUT &H378,(64 OR C%)
   C%=C% OR 64
   MID$(S$,43)="1"
ELSEIF A$="8" THEN              'Port Line 8
   OUT &H378,(128 OR C%)
   C%=C% OR 128
   MID$(S$,49)="1"
END IF

GOTO 10				'Loop back to start

'--------------------------------------------------------------------'
' TURN OFF THE PORT LINE                                             '
' AND the new output with previous output value and also update the  '
' S$ string used to display the Port Line status on the screen       '
'--------------------------------------------------------------------'

30 IF A$="1" THEN               'AND outputs to create new value
   OUT &H378,(254 AND C%)       'Output to Port
   C%=C% AND 254                'Update the previous value
   MID$(S$,7)="0"              'Update the S$ display string
ELSEIF A$="2" THEN              'Port Line 2
   OUT &H378,(253 AND C%)
   C%=C% AND 253
   MID$(S$,13)="0"
ELSEIF A$="3" THEN              'Port Line 3
   OUT &H378,(251 AND C%)
   C%=C% AND 251
   MID$(S$,19)="0"
ELSEIF A$="4" THEN              'Port Line 4
   OUT &H378,(247 AND C%)
   C%=C% AND 247
   MID$(S$,25)="0"
ELSEIF A$="5" THEN              'Port Line 5
   OUT &H378,(239 AND C%)
   C%=C% AND 239
   MID$(S$,31)="0"
ELSEIF A$="6" THEN              'Port Line 6
   OUT &H378,(223 AND C%)
   C%=C% AND 223
   MID$(S$,37)="0"
ELSEIF A$="7" THEN              'Port Line 7
   OUT &H378,(191 AND C%)
   C%=C% AND 191
   MID$(S$,43)="0"
ELSEIF A$="8" THEN              'Port Line 8
   OUT &H378,(127 AND C%)
   C%=C% AND 127
   MID$(S$,49)="0"
END IF

GOTO 10				'Loop back to start

'--------------------------------------------------------------------'
' TURN OFF ALL THE PORT LINES                                        '
'--------------------------------------------------------------------'

40 OUT &H378,0			'Turn off all Port Lines
C% = 0
'
'RESET OUTPUT STATUS STRING
'
S$="   |  0  |  0  |  0  |  0  |  0  |  0  |  0  |  0  |"

GOTO 10				'Loop back to start

50 OUT &H378,0			'Turn off all Port Lines before quiting

'---------------------------------------------------------------------
' TURN TIMER OFF
'---------------------------------------------------------------------

TIMER OFF

'---------------------------------------------------------------------
' TURN KEY CHECKING BACK ON
'---------------------------------------------------------------------

CLS

GOSUB SIGNON			'DISPLAY THE SIGNON SCREEN AGAIN

KEY ON

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON

RETURN

'======================================================================
'			READINPUTS SUBROUTINE
'======================================================================
'
' Parallel Card Address Assignments
' ---------------------------------
'
' Base      - Output bits 0 to 7
'
'             0 - D0         ------> Output 1
'             1 - D1         ------>
'             2 - D2         ------>
'             3 - D3         ------>           Lego Control
'             4 - D4         ------>           Board Output
'             5 - D5         ------>
'             6 - D6         ------>
'             7 - D7         ------> Output 8
'
' Base + 1  - Input bits 3 to 7
'
'             0 - not used
'             1 - not used
'             2 - not used
'  DBPIN #    3 - Error
'  -   13     4 - SLCTD      <------- Input 2  --+
'  -   12     5 - PE         <------- Input 3    |
'  -   10     6 - ACK        <------- Input 5    |
'      11     7 - Busy       <------- Input 4    |
' Base + 2  -                                    +-- Lego Control
'             0 - Strobe                         |   Board Inputs
'             1 - Auto FD XT                     |
'   DBPIN #   2 - Init                           |
'      17     3 - SLCT IN    <------- Input 1  --+
'             4 - IRQ Enable
'             5 - not used
'             6 - not used
'             7 - not used
'
' Pins 10,12,& 13 are inverted on the Parallel Port card.
' Pins 11,& 17 are not inverted on the parallel Port card.
'
'
'
'
READINPUTS:

'----------------------------------------------------------------------
' READ IN INPUTS AND COMBINE INTO ONE WORD
'----------------------------------------------------------------------

   PORTINPUT1% = INP(&H379)		'Read input bits 2 to 5
   PORTINPUT2% = INP(&H37A)		'Read input bit 1
'*************************
' For Testing Only !!!!
'*************************
' SHOW THE BINARY STRING VALUE
'
'    PRINT "379H ",BIN$(PORTINPUT1%),"37AH ",BIN$(PORTINPUT2%)

'   PRINT "ADDRESS 379H ",PORTINPUT1%,"ADDRESS 37AH ",PORTINPUT2%
'   print "hello"

   PORTINPUT1% = PORTINPUT1% AND &H00F0 'mask for inputs 2 to 5
   PORTINPUT2% = PORTINPUT2% AND &H008  'mask input 1
   PORTINPUT% = PORTINPUT1% OR PORTINPUT2%	'combine all bits

'PRINT PORTINPUT1%

'----------------------------------------------------------------------
' SPLIT BITS INTO INDIVIDUAL VALUES
'----------------------------------------------------------------------

   INPUT0% = PORTINPUT% AND &H0008	'Bit 0
   INPUT1% = PORTINPUT% AND &H0016	'Bit 1
   INPUT2% = PORTINPUT% AND &H0032	'Bit 2
   INPUT3% = PORTINPUT% AND &H0128	'Bit 3
   INPUT4% = PORTINPUT% AND &H0064	'Bit 4

'PRINT INPUT0% INPUT1% INPUT2% INPUT3% INPUT4%

'----------------------------------------------------------------------
' NOW UPDATE THE STATUS STRING WITH INPUT BIT VALUES
'----------------------------------------------------------------------

   IF INPUT0% = 1 GOTO SET0
   MID$(SI$,7)="0"              'Update INPUT 0 to 0 in SI$ display string
   GOTO DONE0
SET0:
   MID$(SI$,7)="1"              'Update INPUT 0 to 1 in SI$ display string
DONE0:
   IF INPUT1% = 1 GOTO SET1
   MID$(SI$,13)="0"              'Update INPUT 1 to 0 in SI$ display string
   GOTO DONE1
SET1:
   MID$(SI$,13)="1"              'Update INPUT 1 to 1 in SI$ display string
DONE1:
   IF INPUT2% = 1 GOTO SET2
   MID$(SI$,19)="0"              'Update INPUT 2 to 0 in SI$ display string
   GOTO DONE2
SET2:
   MID$(SI$,19)="1"              'Update INPUT 2 to 1 in SI$ display string
DONE2:
   IF INPUT3% = 1 GOTO SET3
   MID$(SI$,25)="0"              'Update INPUT 3 to 0 in SI$ display string
   GOTO DONE3
SET3:
   MID$(SI$,25)="1"              'Update INPUT 3 to 1 in SI$ display string
DONE3:
   IF INPUT4% = 1 GOTO SET4
   MID$(SI$,31)="0"              'Update INPUT 4 to 0 in SI$ display string
   GOTO DONE4
SET4:
   MID$(SI$,31)="1"              'Update INPUT 4 to 1 in SI$ display string
DONE4:
   LOCATE 8,1
   PRINT SI$,s1$

RETURN

'======================================================================
'			SPAREKEY SUBROUTINE
'======================================================================

SPAREKEY:
	PRINT "SPAREKEY SUBROUTINE"
RETURN

'======================================================================
'			HELP SUBROUTINE
'======================================================================

HELP:
	PRINT "HELP SUBROUTINE"

'---------------------------------------------------------------------
' TURN KEY CHECKING OFF
'---------------------------------------------------------------------

KEY OFF

KEY (1) OFF			'TURN OFF KEY CHECKING
KEY (2) OFF
KEY (3) OFF
KEY (4) OFF
KEY (5) OFF
KEY (6) OFF
KEY (7) OFF
KEY (8) OFF
KEY (9) OFF
KEY (10) OFF

'---------------------------------------------------------------------
' PRINT OUT HELP SCREEN
'---------------------------------------------------------------------

CLS
PRINT" **** THIS IS THE LEGO CONTROL PROGRAM HELP SCREEN ****"
PRINT"============================================================="
PRINT"LEGO COMMAND      DESCRIPTION"
PRINT"-------------------------------------------------------------"
PRINT" ON #            - TURN ON OUTPUT #"
PRINT" OFF #           - TURN OFF OUTPUT #"
PRINT" WAIT #          - WAIT # SECONDS"
PRINT" IF SW # ON *    - IF INPUT # ON TURN OUTPUT * ON"
PRINT" IF SW # OFF *   - IF INPUT # ON TURN OUTPUT * OFF"
PRINT" IF SW # SKIP X  - IF INPUT # ON SKIP X LINES"
PRINT" SKIP X          - SKIP X LINES"
PRINT" END             - END OF PROGRAM"
PRINT" "
PRINT" "
PRINT" "
PRINT" "
PRINT" "
PRINT" "
PRINT" "
PRINT" "
PRINT" "
PRINT" "

LOCATE 23,1
INPUT "PRESS ANY KEY TO CONTINUE "A$

'---------------------------------------------------------------------
' TURN KEY CHECKING BACK ON
'---------------------------------------------------------------------

CLS

GOSUB SIGNON			'DISPLAY THE SIGNON SCREEN AGAIN

KEY ON

KEY (1) ON			'TURN ON KEY CHECKING
KEY (2) ON
KEY (3) ON
KEY (4) ON
KEY (5) ON
KEY (6) ON
KEY (7) ON
KEY (8) ON
KEY (9) ON
KEY (10) ON

RETURN

'======================================================================
'			SETUP SUBROUTINE
'======================================================================

SETUP:
	PRINT "SETUP SUBROUTINE"
RETURN

'======================================================================
'			QUIT SUBROUTINE
'======================================================================

QUIT:
	PRINT "QUIT SUBROUTINE"
	END
RETURN

STOP









